/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * GNUNET MAIN. This is the server-process that must run on each
 * and every GNUNET node. The code in here initializes the subsystems
 * and then dispatches incomming UDP packets using multiple threads
 * to the handler(s).
 *
 * @file server/gnet.c
 * @author Christian Grothoff
 * @author Larry Waldo
 * @author Tzvetan Horozov
 **/

#include "config.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <pthread.h>
 
#include "server/udpserver.h"
#include "server/startup.h"
#include "util/ipcheck.h"
#include "util/semaphore.h"
#include "statistics.h"

/* threads and the shared buffers */
static pthread_t threads_[THREAD_COUNT];

static char buffers[QUEUE_LENGTH][MESSAGE_SIZE];
static int bufferQueue_[QUEUE_LENGTH];
static int bq_firstFree_;
static int bq_lastFree_;
static int bq_firstFull_;

static Semaphore * bufferQueueRead_;
static Semaphore * bufferQueueWrite_;   
static Mutex globalLock_;

static int udp_sock;


/**
 * This is the main loop of each thread.
 * It loops *forever* waiting for incomming packets
 * in the packet queue. Then it calls 
 * "handle" (defined in handler.c) on the packet.
 **/
static void threadMain(int id) {
  int current;
  int needed = 1;
  
  while (needed) {
    LOOPPRINT("threadMain");
    semaphore_down(bufferQueueRead_);
    /* handle buffer entry */
    /* sync with other handlers to get buffer */
    MUTEX_LOCK(&globalLock_);
    current = bufferQueue_[bq_firstFull_++];
    if (bq_firstFull_ == QUEUE_LENGTH)
      bq_firstFull_ = 0;
    MUTEX_UNLOCK(&globalLock_);
    /* end of sync */

    /* handle buffer - now out of sync */    
    handle(id,(MessagePack*)buffers[current]);

    /* free buffer */
    MUTEX_LOCK(&globalLock_);
    bufferQueue_[bq_lastFree_++] = current;
    if (bq_lastFree_ == QUEUE_LENGTH)
      bq_lastFree_ = 0;
    MUTEX_UNLOCK(&globalLock_);
    /* buffer is back in pool. */
    /* handling done - open buffer for write */    
    semaphore_up(bufferQueueWrite_);    
  }
} /* end of threadMain */

/**
 * Listen on the given socket and distribute the packets
 * to the UDP handler.
 * This method never returns.
 **/
static void listenAndDistribute(int sock) {
  int current;
  struct sockaddr_in incoming;
  socklen_t addrlen = sizeof(incoming);  
  int size;

  bzero(&incoming,sizeof(struct sockaddr_in));
  while (1) {
    LOOPPRINT("main (1)");
    semaphore_down(bufferQueueWrite_);
    /* aquire buffer */
    MUTEX_LOCK(&globalLock_);
    if (bq_firstFree_ == QUEUE_LENGTH)
      bq_firstFree_ = 0;
    current = bufferQueue_[bq_firstFree_++];    
    MUTEX_UNLOCK(&globalLock_);
    DEBUG("UDP: Receiving from socket %d, buffer %d.\n",
	  sock, current);
    size = recvfrom(sock, buffers[current], MESSAGE_SIZE, 0,
		    (struct sockaddr * )&incoming, &addrlen);
    GNUNET_STATISTICS.octets_total_udp_in+=size;
    DEBUG("Received %d bytes from %d.%d.%d.%d.\n",
	  size,
	  ((*(int*)&incoming.sin_addr) >> 0) & 255, 
	  ((*(int*)&incoming.sin_addr) >> 8) & 255, 
	  ((*(int*)&incoming.sin_addr) >>16) & 255, 
	  ((*(int*)&incoming.sin_addr) >>24) & 255);
    /* quick test of the packet, if failed, repeat! */
    while ( (size != ((ntohs(((MessagePack*)buffers[current])->size)) & 
		      (ENCRYPTED_FLAG-1))) ||
	    (size < sizeof(MessagePack)) ) {
      LOOPPRINT("main (2)");
      
      WARNING("Received invalid packet (%d != %d) from %d.%d.%d.%d. Discarded.\n",
	    size,
	    (ntohs(((MessagePack*)buffers[current])->size))
	    & (ENCRYPTED_FLAG-1),
	    ((*(int*)&incoming.sin_addr) >> 0) & 255, 
	    ((*(int*)&incoming.sin_addr) >> 8) & 255, 
	    ((*(int*)&incoming.sin_addr) >>16) & 255, 
	    ((*(int*)&incoming.sin_addr) >>24) & 255);
      size = recvfrom(sock, 
		      buffers[current], 
		      MESSAGE_SIZE, 
		      0,
		      (struct sockaddr * )&incoming, 
		      &addrlen);
      GNUNET_STATISTICS.octets_total_udp_in+=size;
      DEBUG("Received %d bytes from %d.%d.%d.%d.\n",
	    size,
	    ((*(int*)&incoming) >> 0) & 255, 
	    ((*(int*)&incoming) >> 8) & 255, 
	    ((*(int*)&incoming) >>16) & 255, 
	    ((*(int*)&incoming) >>24) & 255);
    }
    DEBUG("UDP: Received %d bytes from socket %d, buffer %d.\n",
	  size, sock, current);
   /* finished with list */
    semaphore_up(bufferQueueRead_);
    bzero(&incoming,
	  sizeof(struct sockaddr_in));
  }
}

void initUDPServer() {
  int i;

   /* initialize UDP network */
  udp_sock = passivesock(htons(getGNUnetPort()));
  
  /* initialize sync mechanisms for UDP threads */
  bufferQueueRead_ = new_semaphore(0);
  bufferQueueWrite_ = new_semaphore(QUEUE_LENGTH);
  create_mutex(&globalLock_);
  for (i=0;i<QUEUE_LENGTH;i++)
    bufferQueue_[i]=i;
  bq_firstFree_ = 0;
  bq_lastFree_ = 0;
  bq_firstFull_ = 0;
  
  /* create UDP threads */
  for (i=0;i<THREAD_COUNT;i++) 
    pthread_create(&threads_[i], NULL,
		   (void * (*)(void *)) &threadMain,
		   (void *) &i);  

}

void runUDPDispatch() {
  /* listen & distribute UDP packets to threads */
  listenAndDistribute(udp_sock);
}

void doneUDPServer() {
  close(udp_sock);
}

/* end of udpserver.c */
